import { call, put, takeEvery } from 'redux-sagas/effects'

/** genStuff
  * you input the name of a module (e.g. project) and the API (e.g. http://localhost/db/project)
  * it outputs a state, action types, action creators, reducer, workers and watchers
  * to handle a restful API.
  */
export function reduxLocalstorageGen (name) {
  const actionTypes = {}
  const actions = {}

  /** Supported actions */
  const actionList = ['save', 'restore']

  /** For each supported action create an action creator to request
    * data from the API, a success action when data is processed, and
    * a failure action, when the API call was not successful.
    * You will get e.g. create_request, update_success... */
  actionList.forEach(action => {
    /** generate sth like 'createProject' which dispatches 'project/CREATE_REQUEST' */
    const actionName = `${action}${name[0].toUpperCase()}${name.substring(1)}`
    actions[actionName] = (id, data) => { return { type: `${name}/${action}_REQUEST`, id, data } }
    ['request', 'success', 'failure'].forEach(mode => {
      /** generate sth like 'project/CREATE_REQUEST' */
      const actionType = `${action}_${mode}`.toUpperCase()
      actionTypes[actionType] = `${name}/${actionType}`
    })
  })
  console.log('RESTGen: actionTypes, actions', actionTypes, actions)

  const state = []
  console.log('RESTGen: state', state)

  function * worker (action) {
    try {
      const data = yield call(api, payload)
      yield put({ type: '??_SUCCESS', data })
    } catch (error) {
      yield put({ type: '??_FAILURE', error })
    }
  }

  function * watcher () {
    for (action in actionList) {
      yield takeEvery(`${name}/${action.toUpperCase()}_REQUEST`, worker)
    }
  }

  function reducer (state = [], action) {
    let nextState
    switch (action.type) {
      // On success,
      case actionTypes[`CREATE_REQUEST_${name.toUpperCase()}`]:
        worker(action)
        return nextState
      case actionTypes[`CREATE_SUCCESS_${name.toUpperCase()}`]:
        nextState = [ ...state, action.data ]
        return nextState
      case actionTypes[`UPDATE_REQUEST_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), { ...state[action.id], ...action.data }, ...state.slice(action.id + 1) ]
        return nextState
      case actionTypes[`UPDATE_SUCCESS_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), { ...state[action.id], ...action.data }, ...state.slice(action.id + 1) ]
        return nextState
      case actionTypes[`REMOVE_REQUEST_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), ...state.slice(action.id + 1) ]
        return nextState
      case actionTypes[`REMOVE_SUCCESS_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), ...state.slice(action.id + 1) ]
        return nextState
      default:
        return state
    }
  }

  return {
    actionTypes,
    actions,
    state,
    reducer,
    worker: (sync) ? null : worker,
    watcher: (sync) ? null : watcher
  }
}
